# Music Applet
# Copyright (C) 2007 Paul Kuliniewicz <paul@kuliniewicz.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02111-1301, USA.

import musicapplet.defs
import musicapplet.player

import gobject
from gtk import gdk


INFO = {
        "name": "Exaile",
        "version": musicapplet.defs.VERSION,
        "icon-name": "exaile",
        "author": "Paul Kuliniewicz <paul@kuliniewicz.org>",
        "copyright": "(C) 2007 Paul Kuliniewicz",
        "website": "http://www.kuliniewicz.org/music-applet/",

}


class ExailePlugin (musicapplet.player.DBusPlugin):

    def __init__ (self, conf):
        musicapplet.player.DBusPlugin.__init__ (self, conf,
                                                INFO["name"], INFO["icon-name"],
                                                "org.exaile.DBusInterface")
        self.__player = None
        self.__art_path = None
        self.__poll_source = None


    def _connect (self):
        if self.__player is not None:
            return

        import dbus
        bus = dbus.SessionBus ()

        proxy = bus.get_object ("org.exaile.DBusInterface", "/DBusInterfaceObject")
        self.__player = dbus.Interface (proxy, "org.exaile.DBusInterface")

        self.__poll ()
        self.connected = True
        self.__poll_source = gobject.timeout_add (1000, self.__poll)


    def _disconnect (self):
        self.__player = None

        if self.__poll_source is not None:
            gobject.source_remove (self.__poll_source)
            self.__poll_source = None

        self._set_no_song ()
        self.connected = False


    def toggle_playback (self):
        self.__player.play (reply_handler=lambda: None,
                            error_handler=self._report_dbus_error)


    def previous (self):
        self.__player.prev_track (reply_handler=lambda: None,
                                  error_handler=self._report_dbus_error)


    def next (self):
        self.__player.next_track (reply_handler=lambda: None,
                                  error_handler=self._report_dbus_error)


    # rate_song not supported


    def __poll (self):
        self.__player.query (reply_handler=self.__set_playing,
                             error_handler=self._report_dbus_error)

        self.__player.get_title (reply_handler=self.__set_title,
                                 error_handler=self._report_dbus_error)

        self.__player.get_artist (reply_handler=self.__set_artist,
                                  error_handler=self._report_dbus_error)

        self.__player.get_album (reply_handler=self.__set_album,
                                 error_handler=self._report_dbus_error)

        self.__player.get_cover_path (reply_handler=self.__set_art,
                                      error_handler=self._report_dbus_error)

        self.__player.get_length (reply_handler=self.__set_duration,
                                  error_handler=self._report_dbus_error)

        self.__player.current_position (reply_handler=self.__set_elapsed,
                                        error_handler=self._report_dbus_error)

        return True


    def __set_playing (self, query_result):
        self.playing = query_result.startswith ("status: playing")


    def __set_title (self, title):
        if title != "":
            self.title = title
        else:
            self.title = None


    def __set_artist (self, artist):
        if artist != "":
            self.artist = artist
        else:
            self.artist = None


    def __set_album (self, album):
        if album != "":
            self.album = album
        else:
            self.album = None


    def __set_art (self, path):
        if path != "images/nocover.png" and path != self.__art_path:
            self.__art_path = path
            self.art = gdk.pixbuf_new_from_file (path)
        elif path == "images/nocover.png" and self.__art_path is not None:
            self.__art_path = None
            self.art = None


    def __set_duration (self, duration):
        if duration != "":
            # Assume H:MM:SS or MM:SS format
            parts = duration.split (":")
            result = 0
            for part in parts:
                result = result * 60 + int (part)
            self.duration = result
        else:
            self.duration = -1


    def __set_elapsed (self, percentage):
        if self.duration >= 0:
            self.elapsed = self.duration * percentage / 100;
        else:
            self.elapsed = -1


def create_instance (conf):
    return ExailePlugin (conf)
